defmodule Test16.CLI do
  @moduledoc """
  synopsis:
    Print items from XML document.
  usage:
    $ ./test16 {options} <command> <in-file> <pattern>
  options:
    --verbose     Add more info.
    --count=n     Print n times.
  commands:
    get_elements --        Retrieve and display Xmerl records.
    show_tags --           Show the tags for elements
                               that match a pattern.
    show_text --           Show the text content for elements
                               that match a pattern.
    show_namespace_defs -- Show the name space definitions from elements
                               that match a pattern.
    show_element_count --  Show a count of the number of elements that
                               that match a pattern.
    show_attributes --     Show the attributes and values for elements that
                               that match a pattern.
  """

  @cmd_spec [
    "get_elements",
    "show_tags",
    "show_text",
    "show_namespace_defs",
    "show_element_count",
    "show_attributes",
  ]

  def main([]) do
    IO.puts(@moduledoc)
  end
  def main([help_opt]) when help_opt == "-h" or help_opt == "--help" do
    IO.puts(@moduledoc)
  end
  def main(args) do
    {opts, positional_args, errors} =
      args
      |> parse_args
    case errors do
      [] ->
        process_args(positional_args, opts)
      _ ->
        IO.puts("Bad option:")
        IO.inspect(errors)
        IO.puts(@moduledoc)
    end
  end

  defp parse_args(args) do
    {opts, cmd_and_args, errors} =
      args
      |> OptionParser.parse(strict:
        [verbose: :boolean, ])
    {opts, cmd_and_args, errors}
  end

  defp process_args(positional_args, opts) do
    [command, infile, pattern | _rest] = positional_args
    case command do
      "get_elements" ->
        get_elements(infile, pattern, opts)
      "show_tags" ->
        show_tags(infile, pattern, opts)
      "show_text" ->
        show_text(infile, pattern, opts)
      "show_namespace_defs" ->
        show_namespace_defs(infile, pattern, opts)
      "show_element_count" ->
        show_element_count(infile, pattern, opts)
      "show_attributes" ->
        show_attributes(infile, pattern, opts)
      _ ->
        IO.puts("bad command -- \"#{command}\".")
        IO.puts("Should be one of the following:")
        IO.puts("    #{Enum.join(@cmd_spec, ", ")}")
        IO.puts("")
        IO.puts(@moduledoc)
        :error
    end
  end

  defp get_elements(infile, pattern, _opts) do
    {:ok, elements} = Test16.get_elements(infile, pattern)
    IO.puts("elements:")
    IO.inspect(elements)
  end

  defp show_tags(infile, pattern, _opts) do
    Test16.show_tags(infile, pattern)
  end

  defp show_text(infile, pattern, _opts) do
    Test16.show_text(infile, pattern)
  end

  defp show_namespace_defs(infile, pattern, _opts) do
    Test16.show_namespace_defs(infile, pattern)
  end

  defp show_attributes(file_path, pattern, _opts) do
    Test16.show_attributes(file_path, pattern)
  end

  defp show_element_count(file_path, pattern, _opts) do
    elements = Test16.get_elements(file_path, pattern)
    count = length(elements)
    IO.puts("count: #{count}")
    :ok
  end

end
